@@ -1,12 +1,10 @@
-# @(#)$Ident: FormWidgets.pm 2013-08-15 21:39 pjf ;
-
 package HTML::FormWidgets;
 
 use 5.01;
 use strict;
 use warnings;
-use version; our $VERSION = qv( sprintf '0.21.%d', q$Rev: 1 $ =~ /\d+/gmx );
-use parent       qw( Class::Accessor::Fast );
+use version; our $VERSION = qv( sprintf '0.22.%d', q$Rev: 1 $ =~ /\d+/gmx );
+use parent 'Class::Accessor::Fast';
 
 use Class::Load  qw( is_class_loaded load_class );
 use English      qw( -no_match_vars );
@@ -20,115 +18,48 @@ my $NB      = '&#160;&#8224;&#160;';
 my $NUL     = q();
 my $SPACE   = '&#160;' x 3;
 my $SPC     = q( );
-my $TTS     = q( ~ );
+my $TTS     = ' ~ ';
 my $OPTIONS = {
-   content_type    => q(text/html),
+   content_type    => 'text/html',
    hidden          => sub { {} },
-   js_object       => q(html_formwidgets),
-   language        => q(en),
-   list_key        => q(items),
+   js_object       => 'html_formwidgets',
+   language        => 'en',
+   l10n            => undef,
+   list_key        => 'items',
    literal_js      => sub { [] },
-   ns              => q(default),
+   max_pwidth      => undef,
+   ns              => 'default',
    optional_js     => sub { [] },
    pwidth          => 30,
-   skip            => sub { return { qw(options 1 id 1 name 1 type 1) } },
+   skip            => sub { return { qw( options 1 id 1 name 1 type 1 ) } },
+   uri_for         => undef,
    width           => 1000, };
 my $ATTRS   = {
-   check_field     => undef,        class           => $NUL,
-   clear           => $NUL,         container       => 1,
-   container_class => q(container), container_id    => undef,
-   default         => undef,        frame_class     => $NUL,
-   hacc            => undef,        id              => undef,
-   name            => undef,        onblur          => undef,
-   onchange        => undef,        onkeypress      => undef,
-   options         => undef,        pclass          => q(prompt),
-   pwidth          => undef,        prompt          => $NUL,
-   readonly        => 0,            required        => 0,
-   sep             => undef,        stepno          => undef,
-   text            => $NUL,         tip             => $NUL,
-   tiptype         => q(dagger),    type            => undef, };
+   check_field     => undef,       class           => $NUL,
+   clear           => $NUL,        container       => 1,
+   container_class => 'container', container_id    => undef,
+   default         => undef,       frame_class     => $NUL,
+   hacc            => undef,       hint_title      => undef,
+   id              => undef,       name            => undef,
+   onblur          => undef,       onchange        => undef,
+   onkeypress      => undef,       options         => undef,
+   pclass          => 'prompt',    pwidth          => undef,
+   prompt          => $NUL,        readonly        => 0,
+   required        => 0,           sep             => undef,
+   stepno          => undef,       text            => $NUL,
+   tip             => $NUL,        tiptype         => 'dagger',
+   type            => undef, };
 
 __PACKAGE__->mk_accessors( keys %{ $ATTRS } );
 
-# Class methods
-sub build {
-   my ($class, $options) = @_; $options ||= {}; my $step = 0;
-
-   my $data = delete $options->{data } ||  [];
-   my $key  = $options->{list_key    } ||= $OPTIONS->{list_key    };
-   my $type = $options->{content_type} ||= $OPTIONS->{content_type};
-
-   $options->{hacc    } ||= HTML::Accessors->new( content_type => $type );
-   $options->{iterator} ||= sub { return ++$step };
-
-   for my $list (grep { $_ and ref $_ eq q(HASH) } @{ $data }) {
-      ref $list->{ $key } eq q(ARRAY) or next; my @stack = ();
-
-      for my $item (@{ $list->{ $key } }) {
-         my $built = __build_widget( $class, $options, $item, \@stack );
-
-         $built and push @stack, $built;
-      }
-
-      $list->{ $key } = \@stack;
-   }
-
-   return;
-}
-
-sub new {
-   my ($self, @args) = @_; my $args = __arg_list( @args );
-
-   # Start with some hard coded defaults;
-   my $new = bless { %{ $ATTRS } }, ref $self || $self;
-
-   # Set minimum requirements from the supplied args and the defaults
-   $new->_bootstrap( $args );
-
-   # Your basic factory method trick
-   my $class = ucfirst $new->type;
-      $class = (q(+) eq substr $class, 0, 1)
-             ? (substr $class, 1) : __PACKAGE__."::${class}";
-
-   $new->_ensure_class_loaded( $class );
-   $new->_init( $args ); # Complete the initialization
-
-   return $new;
-}
-
 # Private functions
-sub __arg_list {
+my $_arg_list = sub {
    my (@args) = @_; $args[ 0 ] or return {};
 
-   return ref $args[ 0 ] eq q(HASH) ? $args[ 0 ] : { @args };
-}
+   return ref $args[ 0 ] eq 'HASH' ? $args[ 0 ] : { @args };
+};
 
-sub __build_widget {
-   my ($class, $options, $item, $stack) = @_; $item or return;
-
-   (ref $item and ref $item->{content} eq q(HASH)) or return $item;
-
-   $item->{content}->{form}
-      and return __form_wrapper( $options, $item, $stack );
-
-   if ($item->{content}->{group}) {
-      $options->{skip_groups} and return;
-      return __group_fields( $options, $item, $stack );
-   }
-
-   my $widget = blessed $item->{content}
-              ? $item->{content}
-              : $item->{content}->{widget}
-              ? $class->new( __inject( $options, $item->{content} ) )
-              : undef;
-
-   $widget or return $item;
-   $widget->frame_class and $item->{class} = $widget->frame_class;
-   $item->{content} = $widget->render;
-   return $item;
-}
-
-sub __collect_items {
+my $_collect_items = sub {
    my ($nitems, $stack) = @_; my $html = $NUL; $nitems or return $NUL;
 
    for (1 .. $nitems) {
@@ -136,48 +67,301 @@ sub __collect_items {
    }
 
    return $html;
-}
+};
 
-sub __form_wrapper {
+my $_form_wrapper = sub {
    my ($options, $item, $stack) = @_; my $content = $item->{content};
 
    my $hacc = $options->{hacc};
-   my $html = __collect_items( $content->{nitems}, $stack );
+   my $html = $_collect_items->( $content->{nitems}, $stack );
+   my $attr = $content->{config} // $content->{attrs} // {};
 
-   $item->{content} = "\n".$hacc->form( $content->{attrs}, "\n".$html );
+   $item->{content} = "\n".$hacc->form( $attr, "\n".$html );
 
    return $item;
-}
+};
 
-sub __group_fields {
+my $_group_fields = sub {
    my ($options, $item, $stack) = @_; my $content = $item->{content}; my $class;
 
    $class = delete $content->{frame_class} and $item->{class} = $class;
 
    my $hacc   = $options->{hacc};
    my $legend = $hacc->legend( $content->{text} );
-   my $html   = __collect_items( $content->{nitems}, $stack );
+   my $html   = $_collect_items->( $content->{nitems}, $stack );
 
    $item->{content} = "\n".$hacc->fieldset( "\n".$legend.$html );
-
    return $item;
+};
+
+my $_inject = sub {
+   return { %{ $_[ 0 ] }, options => $_[ 1 ] };
+};
+
+my $_merge_attributes = sub {
+   my ($dest, $src) = @_;
+
+   for my $k (grep  { not exists $dest->{ $_ } or not defined $dest->{ $_ } }
+              keys %{ $src }) {
+      my $v = $src->{ $k };
+
+      defined $v and ref $v eq 'CODE' and $v = $v->();
+      defined $v and $dest->{ $k } = $v;
+   }
+
+   return $dest;
+};
+
+my $_build_widget = sub {
+   my ($class, $opts, $item, $stack) = @_; $item or return;
+
+   (ref $item and (ref $item->{content} eq 'HASH' or blessed $item->{content}))
+      or return $item;
+
+   my $content = $item->{content};
+
+   $content->{form} and return $_form_wrapper->( $opts, $item, $stack );
+
+   if ($content->{group}) {
+      $opts->{skip_groups} and return;
+      return $_group_fields->( $opts, $item, $stack );
+   }
+
+   my $widget = blessed $content   ? $content
+              : $content->{widget} ? $class->new( $_inject->( $content, $opts ))
+                                   : undef;
+
+   $widget or return $item;
+   $widget->frame_class and $item->{class} = $widget->frame_class;
+   $item->{content} = $widget->render;
+   return $item;
+};
+
+# Private object methods
+my $_bootstrap = sub { # Bare minimum is fields + id to get a useful widget
+   my ($self, $args) = @_;
+
+   for my $attr (grep { exists $args->{ $_ } } qw( id name type )) {
+      $self->$attr( $args->{ $attr } );
+   }
+
+   # Defaults id from name
+   my $id = $self->id; my $name = $self->name; my $type = $self->type;
+
+   if ($id and not $name) {
+      $name = $self->name( $id =~ m{ \. }mx ? (split m{ \. }mx, $id)[  1 ]
+                                            : (split m{ \_ }mx, $id)[ -1 ] );
+   }
+
+   not $id and $name and $id = $self->id( $name ); $args->{options} //= {};
+
+   # We can get the widget type from the config file
+   if (not $type and $id and exists $args->{options}->{fields}) {
+      my $fields = $args->{options}->{fields};
+
+      exists $fields->{ $id } and exists $fields->{ $id }->{type}
+         and $type = $self->type( $fields->{ $id }->{type} );
+   }
+   else { $args->{options}->{fields} //= {} }
+
+   # This is the default widget type if not overidden in the config
+   $type or $type = $self->type( 'textfield' );
+   $name or $self->name( $type );
+   return;
+};
+
+my $_build_hacc = sub {
+   # Now we can create HTML elements like we could with CGI.pm
+   my $self = shift; my $hacc = $self->options->{hacc};
+
+   $hacc or $hacc = HTML::Accessors->new
+      ( { content_type => $self->options->{content_type} } );
+   return $hacc
+};
+
+my $_build_pwidth = sub { # Calculate the prompt width
+   my $self   = shift;
+   my $opts   = $self->options;
+   my $width  = $opts->{width} || 1024;
+   my $pwidth = defined $self->pwidth ? $self->pwidth : $opts->{pwidth};
+
+   if ($pwidth and $pwidth =~ m{ \A \d+ \z }mx) {
+      $pwidth  = int $pwidth * $width / 100;
+      $opts->{max_pwidth} and $pwidth > $opts->{max_pwidth}
+         and $pwidth = $opts->{max_pwidth};
+      $pwidth .= 'px';
+   }
+
+   return $pwidth;
+};
+
+my $_build_sep = sub {
+   my $self = shift; my $sep = $self->sep;
+
+   not defined $sep and $self->prompt   and $sep = $COLON;
+       defined $sep and $sep eq 'space' and $sep = $SPACE;
+       defined $sep and $sep eq 'none'  and $sep = $NUL;
+   return $sep;
+};
+
+my $_init_args = sub {
+   my ($self, $args) = @_; my $skip = $self->options->{skip}; my $v;
+
+   for (grep { not $skip->{ $_ } } keys %{ $args }) {
+      exists $self->{ $_ } and defined ($v = $args->{ $_ }) and $self->$_( $v );
+   }
+
+   return;
+};
+
+my $_init_fields = sub {
+   my ($self, $args) = @_; my $fields = $args->{options}->{fields}; my $id;
+
+   $fields and $id = $self->id and exists $fields->{ $id }
+      and $self->$_init_args( $fields->{ $id } );
+   return;
+};
+
+my $_init_hint_title = sub {
+   my ($self, $args) = @_;
+
+   $args->{hint_title} and return $self->hint_title( $args->{hint_title} );
+
+   return $self->hint_title( $self->loc( 'Hint' ) );
+};
+
+my $_init_options = sub {
+   $_[ 0 ]->options( $_merge_attributes->( $_[ 1 ]->{options}, $OPTIONS ) );
+   return;
+};
+
+my $_next_step = sub {
+   return $_[ 0 ]->options->{iterator}->();
+};
+
+my $_render_field = sub {
+   my $self = shift; my $id = $self->id; my $args = {};
+
+   $id                and $args->{id        }  = $id;
+   $self->name        and $args->{name      }  = $self->name;
+   $self->check_field and $args->{class     }  = 'server';
+   $self->required    and $args->{class     } .= ' required';
+   $self->onblur      and $args->{onblur    }  = $self->onblur;
+   $self->onkeypress  and $args->{onkeypress}  = $self->onkeypress;
+   $self->readonly    and $args->{readonly  }  = 'readonly';
+
+   defined $self->default and $args->{default}  = $self->default;
+
+   my $html = $self->render_field( $args );
+   my $name = $self->options->{name} // $NUL;
+   my $ns   = $self->options->{ns  } // $NUL;
+
+   $self->check_field and $self->add_literal_js( 'server', $id, {
+      args   => "[ '${id}', '${name}', '${ns}' ]", event => "'blur'",
+      method => "'checkField'" } );
+   return $html;
+};
+
+my $_set_error = sub {
+   my ($self, $error) = @_; return $self->text( $error );
+};
+
+my $_build_stepno = sub {
+   my $self = shift; my $stepno = $self->stepno;
+
+   defined $stepno and ref $stepno eq 'HASH' and return $stepno;
+   defined $stepno and $stepno eq 'none'     and return $NUL;
+   defined $stepno and $stepno == -1         and $stepno = $self->$_next_step;
+   defined $stepno and $stepno == 0          and $stepno = $SPACE;
+           $stepno and $stepno ne $SPACE     and $stepno = "${stepno}.";
+   return  $stepno;
+};
+
+my $_ensure_class_loaded = sub {
+   my ($self, $class) = @_; my $error;
+
+   try { load_class( $class ) } catch { $error = $self->$_set_error( $_ ) };
+
+   $error and return;
+
+   is_class_loaded( $class )
+      or ( $self->$_set_error
+           ( "Class ${class} loaded but package undefined" ) and return );
+
+   return bless $self, $class; # Rebless ourself as subclass
+};
+
+my $_init = sub {
+   my ($self, $args) = @_;
+
+   $self->$_init_options   ( $args );
+   $self->init             ( $args ); # Allow subclass to set it's own defaults
+   $self->$_init_fields    ( $args );
+   $self->$_init_args      ( $args );
+   $self->$_init_hint_title( $args );
+   $self->hacc             ( $self->$_build_hacc   );
+   $self->pwidth           ( $self->$_build_pwidth );
+   $self->sep              ( $self->$_build_sep    );
+   $self->stepno           ( $self->$_build_stepno );
+   return;
+};
+
+# Class methods
+sub build {
+   my ($class, $options) = @_; $options //= {}; my $step = 0;
+
+   my $data = delete $options->{data } //  [];
+   my $key  = $options->{list_key    } //= $OPTIONS->{list_key    };
+   my $type = $options->{content_type} //= $OPTIONS->{content_type};
+
+   $options->{hacc    } //= HTML::Accessors->new( content_type => $type );
+   $options->{iterator} //= sub { return ++$step };
+
+   for my $list (grep { $_ and ref $_ eq 'HASH' } @{ $data }) {
+      ref $list->{ $key } eq 'ARRAY' or next; my @stack = ();
+
+      for my $item (@{ $list->{ $key } }) {
+         my $built = $_build_widget->( $class, $options, $item, \@stack );
+
+         $built and push @stack, $built;
+      }
+
+      $list->{ $key } = \@stack;
+   }
+
+   return $data;
 }
 
-sub __inject {
-   return { %{ $_[ 1 ] }, options => $_[ 0 ] };
+sub new {
+   my ($self, @args) = @_; my $args = $_arg_list->( @args );
+
+   # Start with some hard coded defaults
+   my $new = bless { %{ $ATTRS } }, blessed $self || $self;
+
+   # Set minimum requirements from the supplied args and the defaults
+   $new->$_bootstrap( $args );
+
+   # Your basic factory method trick
+   my $class = ucfirst $new->type;
+      $class = ('+' eq substr $class, 0, 1)
+             ? (substr $class, 1) : __PACKAGE__."::${class}";
+
+   $new->$_ensure_class_loaded( $class );
+   $new->$_init( $args ); # Complete the initialization
+
+   return $new;
 }
 
 # Public object methods
 sub add_hidden {
    my ($self, $name, $value) = @_;
 
-   my $key    = $self->options->{list_key} || q(items);
+   my $key    = $self->options->{list_key} || 'items';
    my $hidden = $self->options->{hidden  } || {}; $hidden->{ $key } ||= [];
 
    push @{ $hidden->{ $key } }, {
       content => "\n".$self->hacc->input( {
-         name => $name, type => q(hidden), value => $value } ) };
-
+         name => $name, type => 'hidden', value => $value } ) };
    return;
 }
 
@@ -186,9 +370,9 @@ sub add_literal_js {
 
    ($js_class and $id and $config) or return;
 
-   if (ref $config eq q(HASH)) {
+   if (ref $config eq 'HASH') {
       while (my ($k, $v) = each %{ $config }) {
-         if ($k) { $list and $list .= ', '; $list .= $k.': '.($v || 'null') }
+         if ($k) { $list and $list .= ', '; $list .= "${k}: ".($v || 'null') }
       }
    }
    else { $list = $config };
@@ -197,7 +381,6 @@ sub add_literal_js {
 
    push @{ $self->options->{literal_js} },
       "${obj}.config.${js_class}[ '${id}' ] = { ${list} };";
-
    return;
 }
 
@@ -208,16 +391,12 @@ sub add_optional_js {
    return;
 }
 
-sub hint_title {
-   return $_[ 0 ]->{hint_title} ||= $_[ 0 ]->loc( q(form_hint_title) );
-}
-
 sub inflate {
    my ($self, $args) = @_;
 
-   (defined $args and ref $args eq q(HASH)) or return $args;
+   (defined $args and ref $args eq 'HASH') or return $args;
 
-   return __PACKAGE__->new( __inject( $self->options, $args ) )->render;
+   return __PACKAGE__->new( $_inject->( $args, $self->options ) )->render;
 }
 
 sub init { # Can be overridden in factory subclass
@@ -228,10 +407,10 @@ sub is_xml {
 }
 
 sub loc {
-   my ($self, $text, @rest) = @_; my $opt = $self->options; my $l10n;
+   my ($self, $text, @rest) = @_; my $opts = $self->options; my $l10n;
 
-   if (defined ($l10n = $opt->{l10n})) {
-      my $args = { language => $opt->{language}, ns => $opt->{ns} };
+   if (defined ($l10n = $opts->{l10n})) {
+      my $args = { language => $opts->{language}, ns => $opts->{ns} };
 
       return $l10n->( $args, $text, @rest );
    }
@@ -241,7 +420,7 @@ sub loc {
    # Expand positional parameters of the form [_<n>]
    0 > index $text, $LSB and return $text;
 
-   my @args = $rest[0] && ref $rest[0] eq q(ARRAY) ? @{ $rest[0] } : @rest;
+   my @args = $rest[0] && ref $rest[0] eq 'ARRAY' ? @{ $rest[0] } : @rest;
 
    push @args, map { '[?]' } 0 .. 10;
    $text =~ s{ \[ _ (\d+) \] }{$args[ $1 - 1 ]}gmx;
@@ -251,7 +430,7 @@ sub loc {
 sub render {
    my $self  = shift; $self->type or return $self->text || $NUL;
 
-   my $field = $self->_render_field or return $NUL; my $lead = $NUL;
+   my $field = $self->$_render_field or return $NUL; my $lead = $NUL;
 
    $self->stepno      and $lead .= $self->render_stepno;
    $self->prompt      and $lead .= $self->render_prompt;
@@ -261,19 +440,17 @@ sub render {
 
    $field = $lead.$field;
 
-   $self->container   and $field = $self->render_container( $field );
-
-   $self->clear eq q(left) and $field = $self->hacc->br.$field;
-
-   return "\n".$field;
+   $self->container and $field = $self->render_container( $field );
+   $self->clear eq 'left' and $field = $self->hacc->br.$field;
+   return "\n${field}";
 }
 
 sub render_check_field {
    my ($self, $field) = @_; my $hacc = $self->hacc; my $id = $self->id;
 
-   $field .= $hacc->span( { class => q(hidden), id => $id.q(_ajax) } );
+   $field .= $hacc->span( { class => 'hidden', id => "${id}_ajax" } );
 
-   return $hacc->div( { class => q(field_group) }, $field );
+   return $hacc->div( { class => 'field_group' }, $field );
 }
 
 sub render_container {
@@ -289,24 +466,24 @@ sub render_field {
 
    my $id = $args->{id} || '*unknown id*';
 
-   return $self->_set_error( "No render_field method for field ${id}" );
+   return $self->$_set_error( "No render_field method for field ${id}" );
 }
 
 sub render_prompt {
    my $self = shift; my $args = { class => $self->pclass };
 
-   $self->id and $args->{for} = $self->id and $args->{id} = $self->id.q(_label);
+   $self->id and $args->{for} = $self->id and $args->{id} = $self->id.'_label';
 
-   $self->pwidth and $args->{style} .= 'width: '.$self->pwidth.q(;);
+   $self->pwidth and $args->{style} .= 'width: '.$self->pwidth.';';
 
    return $self->hacc->label( $args, $self->prompt );
 }
 
 sub render_separator {
-   my $self = shift; my $class = q(separator);
+   my $self = shift; my $class = 'separator';
 
-   if ($self->sep eq q(break)) {
-      $class = q(separator_break); $self->sep( $NUL );
+   if ($self->sep eq 'break') {
+      $class = 'separator_break'; $self->sep( $NUL );
    }
 
    return $self->hacc->span( { class => $class }, $self->sep );
@@ -315,202 +492,35 @@ sub render_separator {
 sub render_stepno {
    my $self = shift; my $stepno = $self->stepno;
 
-   ref $stepno eq q(HASH) and return $self->inflate( $stepno );
+   ref $stepno eq 'HASH' and return $self->inflate( $stepno );
 
-   return $self->hacc->span( { class => q(step_number) }, $stepno );
+   return $self->hacc->span( { class => 'step_number' }, $stepno );
 }
 
 sub render_tip {
-   my ($self, $field) = @_; my $hacc = $self->hacc;
+   my ($self, $field) = @_; my $hacc = $self->hacc; my $break = 'EOL';
 
-   (my $tip = $self->tip) =~ s{ \n }{ }gmx;
+   (my $tip = $self->tip) =~ s{ \n }{$break}gmx;
 
    $tip !~ m{ $TTS }mx and $tip = $self->hint_title.$TTS.$tip;
    $tip =~ s{ \s+ }{ }gmx;
 
-   my $args = { class => q(help tips), title => $tip };
+   my $args = { class => 'help tips', title => $tip };
 
-   $self->tiptype eq q(dagger) or return $hacc->span( $args, "\n".$field );
+   $self->tiptype eq 'dagger' or return $hacc->span( $args, "\n${field}" );
 
    $field .= $hacc->span( $args, $NB );
 
-   return $hacc->div( { class => q(field_group) }, "\n".$field );
+   return $hacc->div( { class => 'field_group' }, "\n".$field );
 }
 
-# Private object methods
-sub _bootstrap { # Bare minimum is fields + id to get a useful widget
-   my ($self, $args) = @_;
+sub uri_for {
+   my ($self, $url) = @_; defined $url or return;
 
-   for my $attr (grep { exists $args->{ $_ } } qw(id name type)) {
-      $self->$attr( $args->{ $attr } );
-   }
+   ($url !~ m{ \A http[s]?: }mx and defined $self->options->{uri_for})
+      and return $self->options->{uri_for}->( $url );
 
-   # Defaults id from name
-   my $id = $self->id; my $name = $self->name; my $type = $self->type;
-
-   if ($id and not $name) {
-      $name = $self->name( $id =~ m{ \. }mx ? (split m{ \. }mx, $id)[  1 ]
-                                            : (split m{ \_ }mx, $id)[ -1 ] );
-   }
-
-   not $id and $name and $id = $self->id( $name ); $args->{options} ||= {};
-
-   # We can get the widget type from the config file
-   if (not $type and $id and exists $args->{options}->{fields}) {
-      my $fields = $args->{options}->{fields};
-
-      exists $fields->{ $id } and exists $fields->{ $id }->{type}
-         and $type = $self->type( $fields->{ $id }->{type} );
-   }
-   else { $args->{options}->{fields} ||= {} }
-
-   # This is the default widget type if not overidden in the config
-   $type or $type = $self->type( q(textfield) );
-   $name or $self->name( $type );
-   return;
-}
-
-sub _build_hacc { # Now we can create HTML elements like we could with CGI.pm
-   my $self = shift; my $hacc = $self->options->{hacc};
-
-   $hacc or $hacc = HTML::Accessors->new
-      ( { content_type => $self->options->{content_type} } );
-
-   return $hacc
-}
-
-sub _build_pwidth { # Calculate the prompt width
-   my $self   = shift;
-   my $opts   = $self->options;
-   my $width  = $opts->{width} || 1024;
-   my $pwidth = defined $self->pwidth ? $self->pwidth : $opts->{pwidth};
-
-   $pwidth and $pwidth =~ m{ \A \d+ \z }mx
-      and $pwidth = (int $pwidth * $width / 100).q(px);
-
-   return $pwidth;
-}
-
-sub _build_sep {
-   my $self = shift; my $sep = $self->sep;
-
-   not defined $sep and $self->prompt    and $sep = $COLON;
-       defined $sep and $sep eq q(space) and $sep = $SPACE;
-       defined $sep and $sep eq q(none)  and $sep = $NUL;
-
-   return $sep;
-}
-
-sub _build_stepno {
-   my $self = shift; my $stepno = $self->stepno;
-
-   defined $stepno and ref $stepno eq q(HASH) and return $stepno;
-   defined $stepno and $stepno eq q(none)     and return $NUL;
-   defined $stepno and $stepno == -1          and $stepno = $self->_next_step;
-   defined $stepno and $stepno == 0           and $stepno = $SPACE;
-           $stepno and $stepno ne $SPACE      and $stepno = $stepno.q(.);
-
-   return $stepno;
-}
-
-sub _ensure_class_loaded {
-   my ($self, $class) = @_;
-
-   try {
-      load_class( $class );
-      # Rebless ourself as subclass
-      is_class_loaded( $class ) and return bless $self, $class;
-      $self->_set_error( "Class ${class} loaded but package undefined" );
-   }
-   catch { $self->_set_error( $_ ) };
-
-   return;
-}
-
-sub _init {
-   my ($self, $args) = @_;
-
-   $self->_init_options   ( $args );
-   $self->init            ( $args ); # Allow subclass to set it's own defaults
-   $self->_init_fields    ( $args );
-   $self->_init_args      ( $args );
-   $self->_init_hint_title( $args );
-   $self->hacc            ( $self->_build_hacc       );
-   $self->pwidth          ( $self->_build_pwidth     );
-   $self->sep             ( $self->_build_sep        );
-   $self->stepno          ( $self->_build_stepno     );
-   return;
-}
-
-sub _init_args {
-   my ($self, $args) = @_; my $skip = $self->options->{skip}; my $v;
-
-   for (grep { not $skip->{ $_ } } keys %{ $args }) {
-      exists $self->{ $_ } and defined ($v = $args->{ $_ }) and $self->$_( $v );
-   }
-
-   return;
-}
-
-sub _init_fields {
-   my ($self, $args) = @_; my $fields = $args->{options}->{fields}; my $id;
-
-   $fields and $id = $self->id and exists $fields->{ $id }
-      and $self->_init_args( $fields->{ $id } );
-
-   return;
-}
-
-sub _init_hint_title {
-   $_[ 1 ]->{hint_title} and $_[ 0 ]->hint_title( $_[ 1 ]->{hint_title} );
-   return;
-}
-
-sub _init_options {
-   $_[ 0 ]->options( __merge_attributes( $_[ 1 ]->{options}, $OPTIONS ) );
-   return;
-}
-
-sub _next_step {
-   return $_[ 0 ]->options->{iterator}->();
-}
-
-sub _render_field {
-   my $self = shift; my $id = $self->id; my $args = {};
-
-   $id                and $args->{id        }  = $id;
-   $self->name        and $args->{name      }  = $self->name;
-   $self->check_field and $args->{class     }  = q(server);
-   $self->required    and $args->{class     } .= q( required);
-   $self->default     and $args->{default   }  = $self->default;
-   $self->onblur      and $args->{onblur    }  = $self->onblur;
-   $self->onkeypress  and $args->{onkeypress}  = $self->onkeypress;
-   $self->readonly    and $args->{readonly  }  = q(readonly);
-
-   my $html = $self->render_field( $args );
-
-   $self->check_field and $self->add_literal_js( 'server', $id, {
-      args => "[ '${id}' ]", event => "'blur'", method => "'checkField'" } );
-
-   return $html;
-}
-
-sub _set_error {
-   my ($self, $error) = @_; return $self->text( $error );
-}
-
-sub __merge_attributes {
-   my ($dest, $src, $attrs) = @_; $attrs ||= [ keys %{ $src } ];
-
-   for (grep { not exists $dest->{ $_ } or not defined $dest->{ $_ } }
-            @{ $attrs }) {
-      my $v = $src->{ $_ };
-
-      defined $v and ref $v eq q(CODE) and $v = $v->();
-      defined $v and $dest->{ $_ } = $v;
-   }
-
-   return $dest;
+   return $url;
 }
 
 1;
@@ -525,13 +535,13 @@ HTML::FormWidgets - Create HTML user interface components
 
 =head1 Version
 
-Describes version v0.21.$Rev: 1 $ of L<HTML::FormWidgets>
+Describes version v0.22.$Rev: 1 $ of L<HTML::FormWidgets>
 
 =head1 Synopsis
 
    use HTML::FormWidgets;
 
-   my $widget = HTML::FormWidgets->new( id => q(test) );
+   my $widget = HTML::FormWidgets->new( id => 'test' );
 
    print $widget->render;
    # <div class="container">
@@ -567,10 +577,6 @@ reflect this modules primary use within a L<Catalyst> application):
 Some of the widgets require image files. This attribute is used to
 create the URI for those images
 
-=item C<base>
-
-This is the prefix for our URI
-
 =item C<content_type>
 
 Either C<application/xhtml+xml> which generates XHTML 1.1 or
@@ -683,7 +689,7 @@ Returns true if the content type matches C<xml>
    $message_text = $widget->loc( $message_id, @args );
 
 Use the supplied key to return a value from the C<l10n> object. This
-object was passed to the constructor and should localize the key to
+object was passed to the constructor and should localise the key to
 the required language. The C<@args> list contains parameters to substituted
 in place of the placeholders which have the form C<[_n]>
 
@@ -772,48 +778,52 @@ Markup containing the step number on the form if required
 
 Flyover tooltip field help text
 
+=head3 uri_for
+
+Makes absolute URI from relative paths by calling the supplied function
+
 =head2 Private Methods
 
 =head3 _bootstrap
 
-   $widget->_bootstrap( $args );
+   $widget->$_bootstrap( $args );
 
 Determine the C<id>, C<name> and C<type> attributes of the widget from
 the supplied arguments
 
 =head3 _ensure_class_loaded
 
-   $widget->_ensure_class_loaded( $class );
+   $widget->$_ensure_class_loaded( $class );
 
 Once the factory subclass is known this method ensures that it is loaded
 and then re-blesses the self referential object into the correct class
 
 =head3 _set_error
 
-   $widget->_set_error( $error_text );
+   $widget->$_set_error( $error_text );
 
 Stores the passed error message in the C<text> attribute so that it
 gets rendered in place of the widget
 
 =head2 Private Subroutines
 
-=head3 __arg_list
+=head3 _arg_list
 
-   $args = __arg_list( @args );
+   $args = $_arg_list->( @args );
 
 Accepts either a single argument of a hash ref or a list of key/value
 pairs. Returns a hash ref in either case.
 
-=head3 __form_wrapper
+=head3 _form_wrapper
 
-   $item = __form_wrapper( $options, $item, $stack );
+   $item = $_form_wrapper->( $options, $item, $stack );
 
 Wraps the top C<nitems> number of widgets on the build stack in a C<<
 <form> >> element
 
-=head3 __group_fields
+=head3 _group_fields
 
-   $item = __group_fields( $options, $item, $stack );
+   $item = $_group_fields->( $options, $item, $stack );
 
 Wraps the top C<nitems> number of widgets on the build stack in a C<<
 <fieldset> >> element with a legend
@@ -895,10 +905,8 @@ Default subtype. Like the logfile subtype but without the C<< <pre> >> tags
 =item C<html>
 
 The L</_render> method returns an C<< <iframe> >> element whose C<src>
-attribute is set to C<path>. Paths that begin with C<root> will have
-that replaced with the C<base> attribute value. Paths that do not
-begin with C<http:> will have the C<base> attribute value prepended to
-them
+attribute is set to C<path>. Paths that do not
+begin with C<http:> will passed to L</uri_for>
 
 =item C<logfile>
 
@@ -1244,7 +1252,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2012 Peter Flanigan. All rights reserved
+Copyright (c) 2015 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
@@ -1259,4 +1267,3 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE
 # mode: perl
 # tab-width: 3
 # End:
-
