@@ -1,13 +1,10 @@
-# @(#)$Ident: Template.pm 2013-05-16 14:20 pjf ;
-
 package HTML::FormWidgets::Template;
 
 use strict;
 use warnings;
-use version; our $VERSION = qv( sprintf '0.21.%d', q$Rev: 1 $ =~ /\d+/gmx );
-use parent q(HTML::FormWidgets);
+use parent 'HTML::FormWidgets';
 
-use English qw(-no_match_vars);
+use English qw( -no_match_vars );
 use File::Spec;
 use IO::File;
 
